# Generated by roxygen2: do not edit by hand

S3method(AIC,elife_par)
S3method(BIC,elife_par)
S3method(anova,elife_par)
S3method(coef,elife_par)
S3method(confint,elife_profile)
S3method(deviance,elife_par)
S3method(logLik,elife_par)
S3method(nobs,elife_par)
S3method(plot,elife_ecdf)
S3method(plot,elife_northropcoleman)
S3method(plot,elife_npar)
S3method(plot,elife_par)
S3method(plot,elife_profile)
S3method(plot,elife_tstab)
S3method(plot,elife_tstab_endpoint)
S3method(print,elife_northropcoleman)
S3method(print,elife_npar)
S3method(print,elife_par)
S3method(print,elife_par_test)
S3method(print,elife_profile)
S3method(summary,elife_npar)
S3method(summary,elife_par)
S3method(vcov,elife_par)
export(.censTruncLimits)
export(.censTruncLimitsDtrunc)
export(.check_surv)
export(.turnbull_intervals)
export(autoplot.elife_northropcoleman)
export(autoplot.elife_par)
export(autoplot.elife_profile)
export(autoplot.elife_tstab)
export(autoplot.elife_tstab_endpoint)
export(boxcox_transfo)
export(check_arguments)
export(check_elife_dist)
export(dbeard)
export(dbeardmake)
export(delife)
export(dextgp)
export(dextweibull)
export(dgomp)
export(dgompmake)
export(dgpd)
export(dgppiece)
export(dperks)
export(dperksmake)
export(endpoint.profile)
export(endpoint.tstab)
export(fit_ditrunc_elife)
export(fit_elife)
export(hazard_elife)
export(hazard_fn_elife)
export(hbeard)
export(hbeardmake)
export(helife)
export(hexp)
export(hextgp)
export(hextweibull)
export(hgomp)
export(hgompmake)
export(hgpd)
export(hperks)
export(hperksmake)
export(hweibull)
export(lpost_elife)
export(nc_score_test)
export(nc_test)
export(nll_ditrunc_elife)
export(nll_elife)
export(np_elife)
export(npsurv)
export(pbeard)
export(pbeardmake)
export(pelife)
export(pextgp)
export(pextweibull)
export(pgomp)
export(pgompmake)
export(pgpd)
export(pgppiece)
export(pperks)
export(pperksmake)
export(prof_exp_scale)
export(prof_gp_endpt)
export(prof_gp_scalet)
export(prof_gp_shape)
export(qbeard)
export(qbeardmake)
export(qelife)
export(qextgp)
export(qextweibull)
export(qgomp)
export(qgompmake)
export(qgpd)
export(qgppiece)
export(qperks)
export(qperksmake)
export(r_ditrunc_elife)
export(r_dtrunc_elife)
export(r_ltrc_elife)
export(rbeard)
export(relife)
export(rgppiece)
export(samp2_elife)
export(samp_elife)
export(selife)
export(test_ditrunc_elife)
export(test_elife)
export(tstab)
export(turnbull_intervals)
importFrom("graphics","par")
importFrom("stats","quantile")
importFrom(Rcpp,evalCpp)
importFrom(Rsolnp,solnp)
importFrom(graphics,abline)
importFrom(graphics,arrows)
importFrom(graphics,hist)
importFrom(graphics,lines)
importFrom(graphics,mtext)
importFrom(graphics,rug)
importFrom(rlang,.data)
importFrom(stats,AIC)
importFrom(stats,BIC)
importFrom(stats,anova)
importFrom(stats,approxfun)
importFrom(stats,coef)
importFrom(stats,dbinom)
importFrom(stats,deviance)
importFrom(stats,dexp)
importFrom(stats,dweibull)
importFrom(stats,family)
importFrom(stats,logLik)
importFrom(stats,na.omit)
importFrom(stats,nobs)
importFrom(stats,optim)
importFrom(stats,optimize)
importFrom(stats,pbinom)
importFrom(stats,pchisq)
importFrom(stats,pexp)
importFrom(stats,plot.ecdf)
importFrom(stats,ppoints)
importFrom(stats,predict)
importFrom(stats,pweibull)
importFrom(stats,qbinom)
importFrom(stats,qchisq)
importFrom(stats,qexp)
importFrom(stats,qnorm)
importFrom(stats,qweibull)
importFrom(stats,rexp)
importFrom(stats,rmultinom)
importFrom(stats,runif)
importFrom(stats,rweibull)
importFrom(stats,smooth.spline)
importFrom(stats,uniroot)
importFrom(stats,vcov)
importFrom(stats,weighted.mean)
importFrom(utils,packageVersion)
useDynLib(longevity, .registration=TRUE)
